/*****************************************************************************
 *
 * FILE:	wa.h
 * DESCRIPTION:	Wyvern: associated array library header file
 * DATE:	Thu, Oct 21 2004
 * UPDATED:	Wed, Sep  5 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2004-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: wa.h,v 1.9 2007/09/12 02:16:41 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WA_H
#define	_WA_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WAL_VERSION_NUMBER	0x10300000L
#define	WAL_VERSION		"WAL/1.3"
#define	WAL_VERSION_TEXT	WAL_VERSION " (2007/09/05)"
#define	WAL_VERSION_TEXT_LONG \
	"Wyvern Associated Array Library 1.3, Wed, Sep  5 2007"

/*****************************************************************************/

#define	WA_DEFAULT_SIZE	(0)
typedef struct _AAValue {
  const char *		name;
  struct _AAValue *	next;
} AAValue;

typedef struct _AAHash {
  const char *		key;
  AAValue *		value;
  unsigned int		count;	/* number of values */
  struct _AAHash *	next;
} AAHash;
typedef AAHash *	AArray;

typedef struct {
  size_t	bitmask;	/* bitmask for hash */
  size_t	size;		/* the size of hash table */
  size_t	nums;		/* the number of items in a hash table */
  AArray *	table;
} AAObject;

/* callback interface */
typedef void	(*aa_callback_t)(
		const char *	key,
		const char *	value,
		void *		arg,	/* an arbitary pointer */
		unsigned int	order	/* total count at present */
		);

/*****************************************************************************
 *
 *	Global function declaration
 *
 *****************************************************************************/
extern AAObject *	wa_new(size_t);
extern void		wa_free(AAObject *);
extern const char *	wa_put(AAObject *, const char *, const char *);
extern const char *	wa_get(AAObject *, const char *);
extern AAValue *	wa_put_plus(AAObject *, const char *, const char *);
extern AAValue *	wa_get_plus(AAObject *, const char *);
extern char *		wa_get_all(AAObject *, const char *, const char *);
extern int		wa_delete(AAObject *, const char *);
extern int		wa_flush(AAObject *);
extern int		wa_foreach(AAObject *, aa_callback_t, void *);
extern size_t		wa_get_items(AAObject *);

#endif	/* _WA_H */
