/*****************************************************************************
 *
 * FILE:	ws.h
 * DESCRIPTION:	Wyvern: secure library header file
 * DATE:	Wed, May 21 2003
 * UPDATED:	Tue, Sep 11 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2003-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: ws.h,v 1.21 2007/09/12 02:17:15 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WS_H
#define	_WS_H

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WSL_VERSION_NUMBER	0x10003000L
#define	WSL_VERSION		"WSL/1.0.3"
#define	WSL_VERSION_TEXT	WSL_VERSION " (2007/09/11)"
#define	WSL_VERSION_TEXT_LONG	"Wyvern Secure Library 1.0.3, Tue, Sep 11 2007"

/**/
#ifndef	ws_free
#define	ws_free(x)	if((x)!=NULL)free((x))
#endif	/* ws_free */

/*****************************************************************************
 *
 *	Global function declaration
 *
 *****************************************************************************/

/* secure.c */
extern int	ws_drop_privilege(uid_t uid, gid_t gid);
extern int	ws_revoke_privilege(uid_t uid, gid_t gid);
extern int	ws_grant_privilege(void);
extern char *	ws_sanitize_uri(const char * uri);
extern int	ws_safe_open(const char * filename, int open_flags,
			     struct stat * ret_sbuf, int allow_symlink);
extern FILE *	ws_safe_fopen(const char * filename, const char * mode);
extern void	ws_set_error_func(void (*)(void));
extern void *	ws_malloc(size_t size);
extern void *	ws_calloc(size_t nmemb, size_t size);
extern void *	ws_realloc(void * ptr, size_t size);
extern char *	ws_strdup(const char * str);
extern char *	ws_strndup(const char * str, size_t len);
extern int	ws_asprintf(char **ret, const char *fmt, ...);
extern int	ws_strcmp(const char *, const char *);
extern int	ws_strncmp(const char *, const char *, size_t);
extern int	ws_strcasecmp(const char *, const char *);
extern int	ws_strncasecmp(const char *, const char *, size_t);
extern char *	ws_strncat(char *, const char *, size_t);
extern char *	ws_strncpy(char *, const char *, size_t);
extern int	ws_chmod(const char *, mode_t);
extern int	ws_chown(const char *, uid_t, gid_t);

#endif	/* _WS_H */
