/*****************************************************************************
 *
 * FILE:	wt.h
 * DESCRIPTION:	WTL: header file
 * DATE:	Sun, Mar 23 2003
 * UPDATED:	Wed, Sep  5 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2003-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: wt.h,v 1.98 2007/09/04 17:09:04 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WT_H
#define	_WT_H

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <setjmp.h>
#include <signal.h>
#include <dirent.h>
#include <pwd.h>
#include <grp.h>
#include <termios.h>
#if	WTL_ENABLE_SSL
#include <openssl/ssl.h>
#endif	/* WTL_ENABLE_SSL */

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WTL_VERSION_NUMBER	0x10002000L
#if	WTL_ENABLE_SSL
#define	WTL_VERSION		"WTL-SSL/1.0.2"
#define	WTL_VERSION_TEXT	WTL_VERSION " (2007/09/05)"
#else
#define	WTL_VERSION		"WTL/1.0.2"
#define	WTL_VERSION_TEXT	WTL_VERSION " (2007/09/05)"
#endif	/* WTL_ENABLE_SSL */
#define	WTL_VERSION_TEXT_LONG	"Wyvern Thread Library 1.0.2, Wed, Sep  5 2007"

/*****************************************************************************/

#ifndef	_BOOL_T
#define	_BOOL_T
typedef enum {
  false = 0,
  true	= 1
} bool;
#endif	/* _BOOL_T */

/* cycled list */
typedef struct _wt_clist {
  struct _wt_clist *	next;
  struct _wt_clist *	prev;
  void *		body;
} wt_clist_t;

/* time */
typedef unsigned long long	wt_utime_t;
#define	WT_UTIME_UNIT		(1000000ULL)	/* microsecond */
#define	WT_UTIME_MSEC(msec)	(1000ULL*(msec))
#define	WT_UTIME_SEC(sec)	(WT_UTIME_UNIT*(sec))
#define	WT_UTIME_MIN(min)	(WT_UTIME_UNIT*(min)*60)
#define	WT_UTIME_HOUR(hour)	(WT_UTIME_UNIT*(hour)*60*60)
#define	WT_UTIME_NO_TIMEOUT	(-1ULL)
#define	WT_UTIME_TO_SEC(x)	((x)/WT_UTIME_UNIT)
#define	WT_UTIME_TO_NSEC(x)	((x)*1000ULL)

/* private key */
typedef unsigned int	wt_key_t;
#define	WT_KEYS_MAX	(16)
/* destuctor for private key table */
typedef void		(*wt_destructor_t)(void *);

/* condition variable wait queue */
typedef wt_clist_t *	wt_cond_t;
#define	WT_COND_INITIALIZER	NULL

/* mutex wait queue */
typedef wt_clist_t *	wt_mutex_t;
#define	WT_MUTEX_INITIALIZER	NULL

/* once definition */
typedef struct _wt_once {
  int		state;
  wt_mutex_t	mutex;
} wt_once_t;
/* flags for once initialization */
#define	WT_NEED_INIT	(0)
#define	WT_DONE_INIT	(1)
/* static once initialization values */
#define	WT_THREAD_ONCE_INIT	{ WT_NEED_INIT, WT_MUTEX_INITIALIZER }

/* thread attribution */
typedef struct {
  const char *	name;		/* thread name (useful for debug) */
  bool		joinable;	/* does thread permit join? */
  void *	stackaddr;	/* new signal stack address */
  size_t	stacksize;	/* signal stack size for thread */
} wt_thread_attr_t;
extern const wt_thread_attr_t	wt_thread_attr_defaults;

/* thread state */
typedef enum {
  WT_ST_RUNNABLE,
  WT_ST_RUNNING,
  WT_ST_WAIT_MUTEX,
  WT_ST_WAIT_COND,
  WT_ST_SLEEPING,
  WT_ST_SUSPENDED,
  WT_ST_ZOMBIE,
  WT_ST_DEAD
} wt_thread_state;

/* thread flags */
typedef enum {
  WT_FL_PRIMORDIAL	= 0x01,
  WT_FL_IDLE_THREAD	= 0x02,
  WT_FL_ON_SLEEPQ	= 0x04,
  WT_FL_TIMEDOUT	= 0x08,
  WT_FL_CANCELED	= 0x10,
  WT_FL_WAIT_IO		= 0x20,
  WT_FL_WAIT_PID	= 0x40	/* wait for process termination */
} wt_thread_flags;

/* default signal stack size */
#define	WT_VP_SS_SIZE(n)	((n)*1024 + SIGSTKSZ)
#define	WT_VP_SS_TINY		(4)
#define	WT_VP_SS_MINI		(8)
#define	WT_VP_SS_SMALL		(16)
#define	WT_VP_SS_MEDIUM		(32)
#define	WT_VP_SS_LARGE		(64)
#define	WT_VP_SS_HUGE		(128)
#define	WT_STACK_SIZE		(WT_VP_SS_SIZE(0))

/* machine context data structure (Thread) */
typedef struct _mctx_st {
  const char *		name;
  wt_thread_state	state;	/* thread state */
  wt_thread_flags	flags;	/* thread flags */
  wt_clist_t		q;	/* link on run/sleep/zombie queue */
  wt_clist_t		wait_q;	/* link on mutex/condvar wait queue */
  wt_utime_t		sleep;	/* sleep time */
  wt_cond_t		term;	/* termination condition variable for join */
  void **		data;	/* thread private data */
  void *		retval;	/* return value of thread */
  int			ecode;	/* errno */
  sigset_t		sigs;	/* signal mask to apply */
  void *		ss_addr;/* signal stack with WT_STACK_SIZE */
  size_t		ss_size;/* signal stack size for thread */
  sigjmp_buf		jb;	/* machine context */
} mctx_t;
typedef struct _mctx_st *	wt_thread_t;

/*****************************************************************************/

#define	WT_POLL_NONE	0x0000	/* clear all bits */
#define	WT_POLL_IN	0x0001	/* fd is readable */
#define	WT_POLL_OUT	0x0002	/* fd is writable (won't block) */
#define	WT_POLL_INVAL	0x0004	/* invalid pollfd entry */

/* role arguments for wt_netfd_initport() */
#define	WT_ROLE_CLIENT	0x01
#define	WT_ROLE_SERVER	0x02
#define	WT_PROTOCOL_TCP	0x04
#define	WT_PROTOCOL_UDP	0x08
#define	WT_SOCK_STREAM	WT_PROTOCOL_TCP
#define	WT_SOCK_DGRAM	WT_PROTOCOL_UDP
#define	WT_UNIX_DOMAIN	0x10
#define	WT_TCP_CLIENT	(WT_ROLE_CLIENT|WT_PROTOCOL_TCP)
#define	WT_TCP_SERVER	(WT_ROLE_SERVER|WT_PROTOCOL_TCP)
#define	WT_UDP_CONNECT	0x20
#define	WT_UDP_CLIENT	(WT_ROLE_CLIENT|WT_PROTOCOL_UDP)
#define	WT_UDP_SERVER	(WT_ROLE_SERVER|WT_PROTOCOL_UDP)
#define	WT_UST_CLIENT	(WT_ROLE_CLIENT|WT_UNIX_DOMAIN|WT_SOCK_STREAM)
#define	WT_UST_SERVER	(WT_ROLE_SERVER|WT_UNIX_DOMAIN|WT_SOCK_STREAM)
#define	WT_UDG_CONNECT	0x40
#define	WT_UDG_CLIENT	(WT_ROLE_CLIENT|WT_UNIX_DOMAIN|WT_SOCK_DGRAM)
#define	WT_UDG_SERVER	(WT_ROLE_SERVER|WT_UNIX_DOMAIN|WT_SOCK_DGRAM)
#define	WT_TCP_CONNECT	0x80

/* howto arguments for wt_netfd_shutdown() */
#ifdef	SHUT_RD		/* shut down the reading side */
#define	WT_SHUT_RD	SHUT_RD
#else
#define	WT_SHUT_RD	(0)
#endif	/* SHUT_RD */
#ifdef	SHUT_WR		/* shut down the writing side */
#define	WT_SHUT_WR	SHUT_WR
#else
#define	WT_SHUT_WR	(1)
#endif	/* SHUT_WR */
#ifdef	SHUT_RDWR	/* shut down both sides */
#define	WT_SHUT_RDWR	SHUT_RDWR
#else
#define	WT_SHUT_RDWR	(2)
#endif	/* SHUT_RDWR */


typedef struct _wt_netfd {
#if	WTL_ENABLE_SSL
  SSL_CTX *		ssl_ctx;	/* SSL context */
  SSL *			ssl;		/* SSL connection */
#endif	/* WTL_ENABLE_SSL */
  int			osfd;		/* underlying OS file descriptor */
  bool			inuse;		/* In-use flag */
  struct sockaddr *	saddr;
  socklen_t		addrlen;
  char *		udpath;		/* unix domain socket */
  struct termios	tty;
#define	WT_BUFSIZE	(1024*4)
  struct {	/* read buffer */
    int		total;	/* initialize to 0 */
    char *	bufp;	/* initialize to buf */
    char	buf[WT_BUFSIZE];
  } rb;
  struct _wt_netfd *	next;
} wt_netfd_t;
#if	WTL_ENABLE_SSL
#define	wt_netfd_ssl(nfd)	((nfd)->ssl)
#endif	/* WTL_ENABLE_SSL */
#define	wt_netfd_fileno(nfd)	((nfd)->osfd)
#define	wt_netfd_sockaddr(nfd)	((nfd)->saddr)
#define	wt_netfd_socklen(nfd)	((nfd)->addrlen)
#define	wt_netfd_assign(to,from)	\
	do { \
	  (to)=(from); \
	  (to)->next=NULL; \
	} while (0)

/* set file offset to offset */
#ifdef	SEEK_SET
#define WT_SEEK_SET	SEEK_SET
#else
#define WT_SEEK_SET	0
#endif	/* SEEK_SET */
/* set file offset to current plus offset */
#ifdef	SEEK_CUR
#define WT_SEEK_CUR	SEEK_CUR
#else
#define WT_SEEK_CUR	1
#endif	/* SEEK_CUR */
/* set file offset to EOF plus offset */
#ifdef	SEEK_END
#define WT_SEEK_END	SEEK_END
#else
#define WT_SEEK_END	2
#endif	/* SEEK_END */

/* useful macros for wt_netfd_initport */
#define	wt_netfd_tcp_connect(a,p,t) \
	wt_netfd_initport(a, p, WT_TCP_CONNECT, t)
#define	wt_netfd_tcp_client(a,p) \
	wt_netfd_initport(a, p, WT_TCP_CLIENT, -1)
#define	wt_netfd_tcp_server(a,p,q) \
	wt_netfd_initport(a, p, WT_TCP_SERVER, q)
#define	wt_netfd_udp_client(a,p) \
	wt_netfd_initport(a, p, WT_UDP_CLIENT, -1)
#define	wt_netfd_udp_connect(a,p) \
	wt_netfd_initport(a, p, WT_UDP_CONNECT, -1)
#define	wt_netfd_udp_server(a,p) \
	wt_netfd_initport(a, p, WT_UDP_SERVER, -1)
#define	wt_netfd_unix_stream_client(a) \
	wt_netfd_initport(a, 0, WT_UST_CLIENT, -1)
#define	wt_netfd_unix_stream_server(a,q) \
	wt_netfd_initport(a, 0, WT_UST_SERVER, q)
#define	wt_netfd_unix_datagram_connect(a) \
	wt_netfd_initport(a, 0, WT_UDG_CONNECT, -1)
#define	wt_netfd_unix_datagram_client(a) \
	wt_netfd_initport(a, 0, WT_UDG_CLIENT, -1)
#define	wt_netfd_unix_datagram_server(a) \
	wt_netfd_initport(a, 0, WT_UDG_SERVER, -1)

/* for wt_spawn() */
#define	WT_DEFAULT_CPU_LIMIT	(0)
#define	WT_DEFAULT_FSIZE_LIMIT	(0)

/*****************************************************************************
 *
 *	Global function declaration
 *
 *****************************************************************************/

/* schd.c */
extern int		wt_init(int);
extern int		wt_thread_create(wt_thread_t *,
					 const wt_thread_attr_t *,
					 void * (*)(void *), void *);
extern void		wt_thread_exit(void *);
extern int		wt_thread_detach(wt_thread_t);
extern int		wt_thread_join(wt_thread_t, void **);
extern wt_thread_t	wt_thread_self(void);
extern void		wt_thread_yield(void);
extern int		wt_thread_cancel(wt_thread_t);
extern int		wt_thread_once(wt_once_t *, void (*)(void));
extern void		wt_thread_run(void);

extern wt_utime_t	wt_utime(void);
extern int		wt_usleep(wt_utime_t);
extern int		wt_sleep(int);
/* mutex.c */
extern int		wt_thread_mutex_init(wt_mutex_t *);
extern int		wt_thread_mutex_destroy(wt_mutex_t);
extern int		wt_thread_mutex_lock(wt_mutex_t);
extern int		wt_thread_mutex_trylock(wt_mutex_t);
extern int		wt_thread_mutex_unlock(wt_mutex_t);
/* cond.c */
extern int		wt_thread_cond_init(wt_cond_t *);
extern int		wt_thread_cond_destroy(wt_cond_t);
extern int		wt_thread_cond_broadcast(wt_cond_t);
extern int		wt_thread_cond_signal(wt_cond_t);
extern int		wt_thread_cond_timedwait(wt_cond_t, wt_utime_t);

/* key.c */
extern int		wt_thread_key_create(wt_key_t * key,
					     void (*destructor)(void *));
extern int		wt_thread_key_delete(wt_key_t key);
extern void *		wt_thread_getspecific(wt_key_t key);
extern int		wt_thread_setspecific(wt_key_t key, void *value);
extern void		wt_thread_cleanup(wt_thread_t);

/* io.c */
extern int		wt_netfd_poll(wt_netfd_t *, unsigned short, wt_utime_t);
extern wt_netfd_t *	wt_netfd_open(const char *, int, mode_t);
extern wt_netfd_t *	wt_netfd_fdopen(int);
extern int		wt_netfd_close(wt_netfd_t *);
extern off_t		wt_netfd_seek(wt_netfd_t *, off_t, int);
extern ssize_t		wt_netfd_read0(wt_netfd_t *, void *, size_t,
				       wt_utime_t);
extern ssize_t		wt_netfd_read(wt_netfd_t *, void *, size_t, wt_utime_t);
extern ssize_t		wt_netfd_write(wt_netfd_t *, const void *, size_t,
				       wt_utime_t);
extern wt_netfd_t *	wt_netfd_accept(wt_netfd_t *, struct sockaddr *,
					socklen_t *, wt_utime_t);
extern int		wt_netfd_connect(wt_netfd_t *, const struct sockaddr *,
					 socklen_t, wt_utime_t);
extern ssize_t		wt_netfd_recvfrom(wt_netfd_t *, const void *, size_t,
					  int, struct sockaddr *, socklen_t *,
					  wt_utime_t);
extern ssize_t		wt_netfd_sendto(wt_netfd_t *, const void *, size_t, int,
					const struct sockaddr *, socklen_t,
					wt_utime_t);
#if	HAVE_FREEBSD_SENDFILE
extern int		wt_netfd_sendfile(wt_netfd_t *, wt_netfd_t *, off_t,
					  size_t, struct sf_hdtr *, off_t *,
					  int, wt_utime_t);
#elif	HAVE_LINUX_SENDFILE
extern int		wt_netfd_sendfile(wt_netfd_t *, wt_netfd_t *, off_t,
					  size_t, void *, off_t *, int,
					  wt_utime_t);
#else	/* No sendfile(2) */
extern int		wt_netfd_sendfile(wt_netfd_t *, wt_netfd_t *, off_t,
					  size_t, void *, off_t *, int,
					  wt_utime_t);
#endif
extern ssize_t		wt_netfd_readline(wt_netfd_t *, void *, size_t,
					  wt_utime_t);

/* net.c */
extern wt_netfd_t *	wt_netfd_initport(const char *, const char *, int, int);
extern int		wt_netfd_shutdown(wt_netfd_t *, int);
extern struct addrinfo * wt_host_serv(const char *, const char *, int, int);
extern char *		wt_sock_ntop(const struct sockaddr *, bool);
extern wt_netfd_t *	wt_netfd_tcp_servers(const char *, const char *, int);

/* aio.c */
extern ssize_t		wt_netfd_readn(wt_netfd_t *, void *, size_t,
				       wt_utime_t);
extern ssize_t		wt_netfd_writen(wt_netfd_t *, const void *, size_t,
					wt_utime_t);
extern ssize_t		wt_netfd_recv_fd(wt_netfd_t *, int *, wt_utime_t);
extern ssize_t		wt_netfd_send_fd(wt_netfd_t *, int, wt_utime_t);
extern ssize_t		wt_netfd_readv(wt_netfd_t *, const struct iovec *, int,
				       wt_utime_t);
extern ssize_t		wt_netfd_writev(wt_netfd_t *, const struct iovec *, int,
					wt_utime_t);

/* sio.c */
extern wt_netfd_t *	wt_netfd_initcom(const char *, unsigned int);
extern int		wt_netfd_com_close(wt_netfd_t *);

/* proc.c */
extern int		wt_spawn(const char *, const char *, char * const [],
				 char * const [], void (*)(int), int, int, int,
				 int, int, wt_utime_t);

/* ssl.c */
extern wt_netfd_t *	wt_netfd_ssl_tcp_server(const char *, const char *, int,
						const char *, const char *,
						const char *);
extern wt_netfd_t *	wt_netfd_ssl_tcp_servers(const char *, const char *,
						 int, const char *,
						 const char *, const char *);
extern wt_netfd_t *	wt_netfd_ssl_tcp_client(const char *, const char *);
extern int		wt_netfd_ssl_close(wt_netfd_t *);
extern int		wt_netfd_ssl_shutdown(wt_netfd_t *, int);
extern ssize_t		wt_netfd_ssl_read(wt_netfd_t *, void *, size_t,
					  wt_utime_t);
extern ssize_t		wt_netfd_ssl_write(wt_netfd_t *, const void *, size_t,
					   wt_utime_t);
extern wt_netfd_t *	wt_netfd_ssl_accept(wt_netfd_t *, struct sockaddr *,
					    socklen_t *, wt_utime_t);
#if	HAVE_FREEBSD_SENDFILE
extern int		wt_netfd_ssl_sendfile(wt_netfd_t *, wt_netfd_t *, off_t,
					  size_t, struct sf_hdtr *, off_t *,
					  int, wt_utime_t);
#elif	HAVE_LINUX_SENDFILE
extern int		wt_netfd_ssl_sendfile(wt_netfd_t *, wt_netfd_t *, off_t,
					  size_t, void *, off_t *, int,
					  wt_utime_t);
#else	/* No sendfile(2) */
extern int		wt_netfd_ssl_sendfile(wt_netfd_t *, wt_netfd_t *, off_t,
					  size_t, void *, off_t *, int,
					  wt_utime_t);
#endif
extern ssize_t		wt_netfd_ssl_readline(wt_netfd_t *, void *, size_t,
					      wt_utime_t);
extern ssize_t		wt_netfd_ssl_writev(wt_netfd_t *, const struct iovec *,
					    int, wt_utime_t);

#endif	/* _WT_H */
