/*****************************************************************************
 *
 * FILE:	wu.h
 * DESCRIPTION:	Wyvern: utility library header file
 * DATE:	Tue, May 27 2003
 * UPDATED:	Sun, Aug 19 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2003-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: wu.h,v 1.26 2007/08/19 05:55:41 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WU_H
#define	_WU_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <pwd.h>
#include <grp.h>
#include <time.h>

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WUL_VERSION_NUMBER	0x10100000L
#define	WUL_VERSION		"WUL/1.1"
#define	WUL_VERSION_TEXT	WUL_VERSION " (2007/08/19)"
#define	WUL_VERSION_TEXT_LONG	"Wyvern Utility Library 1.1, Sun, Aug 19 2007"

/* time format */
#define	WUL_TIME_FORMAT_RFC822	(1)
#define	WUL_TIME_FORMAT_RFC850	(2)
#define	WUL_TIME_FORMAT_ANSI_C	(3)
#define	WUL_TIME_FORMAT_APACHE	(4)
#define	WUL_TIME_FORMAT_TYPICAL	(5)
#define	WUL_TIME_FORMAT_SQL	(6)

/* timezone */
#define	WUL_ZONE_GMT	(1)
#define	WUL_ZONE_LOCAL	(0)

/* flag for wu_strsep */
#define	WUL_STRSEP_ORIGINAL	(0)
#define	WUL_STRSEP_EXPANSION	(1)

#define	wu_strsep1(s,d)	wu_strsep((s),(d),WUL_STRSEP_ORIGINAL)
#define	wu_strsep2(s,d)	wu_strsep((s),(d),WUL_STRSEP_EXPANSION)

/*****************************************************************************
 *
 *	Global function declaration
 *
 *****************************************************************************/

/* util.c */
extern char *	wu_chomp(char *);
extern int	wu_close(int);
extern int	wu_skip_line(const char *);
extern char *	wu_strsep(char **, const char *, int);
extern int	wu_timestamp(char ** retbuf, time_t tloc, int format, int gmt);
extern int	wu_uptime(char **, struct timeval *, struct timeval *);
extern char *	wu_uri_escaped(const char *);
extern char *	wu_uri_unescaped(const char *);
extern char *	wu_make_boundary(unsigned long);
extern char *	wu_base64_encode(const char *);
extern char *	wu_base64_decode(const char *);
extern int	wu_mkstemp(const char *);
extern char *	wu_toupper(char *);
extern char *	wu_tolower(char *);
extern char *	wu_dirname(const char *);
extern int	wu_daemon(int, int, const char *);
extern int	wu_store_pid(const char *, uid_t, gid_t);

#endif	/* _WU_H */
