/*****************************************************************************
 *
 * FILE:	wyvern.h
 * DESCRIPTION:	Wyvern: global header file
 * DATE:	Thu, Aug 26 1999
 * UPDATED:	Wed, Sep 12 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wyvern.h.in,v 1.53 2007/09/12 02:24:27 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WYVERN_H
#define	_WYVERN_H

#ifdef	MAIN_MODULE
#define	EXTERN
#else
#define	EXTERN extern
#endif	/* MAIN_MODULE */

#include <sys/types.h>
#include <time.h>
#include <sxml.h>
#include "common.h"
#include "status.h"
#include "module.h"
#if	ENABLE_SSL
#include "wt-ssl.h"
#else
#include "wt.h"
#endif	/* ENABLE_SSL */
#include "wc.h"
#include "wl.h"
#include "ws.h"
#include "wu.h"
#include "wa.h"

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WYVERN_VERSION_NUMBER	0x20210000L
#define	WYVERN_VERSION		"Wyvern/2.2.10"
#define	WYVERN_VERSION_TEXT	WYVERN_VERSION " (2007/09/12)"
#define	WYVERN_URI		SERVER_ADDRESS

#define	SERVER_VERSION \
	WYVERN_VERSION " " \
	WTL_VERSION " " \
	SXML_VERSION
#define	SERVER_SOFTWARE \
	WYVERN_VERSION_TEXT " + " WTL_VERSION_TEXT " + " SXML_VERSION_TEXT
#define	SERVER_ADDRESS	"http://www.MysticWALL.COM/software/wyvern/index.html"

/*
 *	Default Values
 */
#define	DEF_SERVER_ROOT	"/usr/local/wyvern"
#define	DEF_PORT	"80"	/* Port */
#define	DEF_SECURE_PORT	"443"	/* SecurePort via SSL */
#define	DEF_MIN_THREAD	(2)	/* Minimum Thread */
#define	DEF_MAX_THREAD	(8)	/* MaxThread */
#define	DEF_MIN_REQUEST	(2)	/* Minimum Request */
#define	DEF_MAX_REQUEST	(8)	/* MaxRequest */
#define	DEF_MAX_POST_SIZE	(1*1024*1024)	/* file resource limit: 1MB */
#define	DEF_USER	(80)	/* User */
#define	DEF_GROUP	(80)	/* Group */
#define	DEF_LOG_DIR		DEF_SERVER_ROOT "/logs"
#define	DEF_SECURE_LEVEL	(-1)		/* SecureLevel */
#define	DEF_SECURITY_LOG	DEF_LOG_DIR "/security.log" /* SecurityLog */
#define	DEF_PID_FILE		DEF_LOG_DIR "/wyvern.pid"	/* PidFile */
#define	DEF_CONF_DIR	"conf"	/* MUST be relative path */
#define	DEF_ETC_DIR	"etc"	/* MUST be relative path */
#define	DEF_TMP_DIR	DEF_SERVER_ROOT "/tmp"
#define	DEF_CACHE_DIR	DEF_SERVER_ROOT "/cache"
#define	DEF_ERROR_LOG	DEF_LOG_DIR "/error.log"	/* ErrorLog */
#define	DEF_ACCESS_LOG	DEF_LOG_DIR "/access.log"	/* AccessLog */
#define	DEF_TIMEOUT	(30)			/* Timeout (sec.) */
#define	DEF_KA_TIMEOUT	(10)			/* KeepAliveTimeout (sec.) */
#define	DEF_KA_MAX	(15)			/* KeepAliveMax */
#define	DEF_DOC_ROOT	DEF_SERVER_ROOT "/htdocs"	/* DocumentRoot */
#define	DEF_USERDIR	"public_html"		/* UserDir */
#define	DEF_DIR_INDEX	"index.html"		/* DirectoryIndexes */
#define	DEF_SYMLINKS	(0)			/* FollowSymLinks (0:off 1:on)*/
#define	DEF_DEFTYPE	"text/plain"		/* DefaultType */
#define	DEF_WYVERN_CONFIG	DEF_CONF_DIR "/wyvern.conf"
#define	DEF_TYPES_CONFIG	DEF_CONF_DIR "/mime.types"
/**/
#define	WYVERN_CONF_DIR	DEF_SERVER_ROOT "/" DEF_CONF_DIR
#define	WYVERN_DB_DIR	DEF_SERVER_ROOT "/db"

/*
 *	Security Level
 */
#define	MAX_SECURE_LEVEL	(7)		/* SecureLevel */
#define SECURE_LEVEL0		(0)
#define SECURE_LEVEL1		(1)
#define SECURE_LEVEL2		(2)
#define SECURE_LEVEL3		(3)
#define SECURE_LEVEL4		(4)
#define SECURE_LEVEL5		(5)
#define SECURE_LEVEL6		(6)
#define SECURE_LEVEL7		(7)

/*
 *	Paths
 */
#define	FILE_AUTH	"wauth.conf"
#define	FILE_PASSWD	"wpasswd"
#define	FILE_DIGEST	"wdigest"
#define	FILE_ACCESS	"waccess.conf"
/* for users */
#define	DEF_USER_DIR		".wyvern"
#define	WYVERN_USER_AUTH	DEF_USER_DIR "/" FILE_AUTH
#define	WYVERN_USER_PASSWD	DEF_USER_DIR "/" FILE_PASSWD
#define	WYVERN_USER_DIGEST	DEF_USER_DIR "/" FILE_DIGEST
/* for server */
#define	WYVERN_SERVER_ACCESS	FILE_ACCESS
#define	WYVERN_SERVER_AUTH	FILE_AUTH
#define	WYVERN_SERVER_PASSWD	DEF_ETC_DIR "/" FILE_PASSWD
#define	WYVERN_SERVER_DIGEST	DEF_ETC_DIR "/" FILE_DIGEST

/*
 *	Size and Length
 */
#define	ALIAS_TABLE_SIZE	(11)	/* Redirect/Alias/ScriptAlias */
#define	ERRDOC_TABLE_SIZE	(5)	/* customizable error documents */
#define	MIME_TABLE_SIZE		(79)	/* mime types */
#define	USER_TABLE_SIZE		(13)	/* user's home directory cache */

#define	MAX_TIMEOUT		(60*60*24)	/* 24 hour */
#define	MAX_CGI_ENVNO		(34+1)	/* max number of CGI environment */
#define	MAX_URI_LEN		(256)	/* maximum URI length */
#define	MAX_FILENAME_LEN	(80)
#define	MAX_NAME_LEN		(16)
#if	MAXPATHLEN
#define	MAX_PATH_LEN		MAXPATHLEN
#else
#define	MAX_PATH_LEN		(512)
#endif	/* MAXPATHLEN */
#if	MAXHOSTNAMELEN
#define	MAX_HOSTNAME_LEN	MAXHOSTNAMELEN
#else
#define	MAX_HOSTNAME_LEN	(256)
#endif	/* MAXHOSTNAMELEN */

/*
 *	Users/Group
 */
#define	UID_ROOT	(0)	/* super user id in /etc/passwd */
#define	GID_WHEEL	(0)	/* super user's group id in /etc/group */

/*
 *	Magic Types
 */
#define	MAGIC_TYPE_ANY		""
#define	MAGIC_TYPE_HTML		"text/html"
#define	MAGIC_TYPE_PLAIN	"text/plain"
#define	MAGIC_TYPE_CGI		"application/x-httpd-cgi"
#define	MAGIC_TYPE_SSI		"text/x-server-parsed-html"
#define	MAGIC_TYPE_ECGI		"application/x-httpd-embedded-cgi"
#define	MAGIC_TYPE_SUEXEC	"application/x-httpd-suexec"
#define	MAGIC_TYPE_DIR		"httpd/unix-directory"

/*
 *	Misc
 */
#define	DEFAULT_PATH	"/bin:/usr/bin:/usr/local/bin"
#define	CGI_VERSION	"CGI/1.1"
#define	ALLOW_METHODS	"HEAD,GET,POST"
#define	RANGE_UNIT	"bytes"
#define	TILDE_OFFSET	(2)	/* offset of '~' */

/*****************************************************************************/

typedef struct _StrList {
  char *		text;
  struct _StrList *	next;
} StrList;

typedef struct _Hash {
  char *		name;
  char *		value;
  struct _Hash *	next;
} Hash;

typedef struct {	/* logfile structure */
  char *	path;	/* path of log file */
  FILE *	fp;	/* file pointer for log */
} LogFile;

/* Error document type */
typedef enum {
  ED_STRING,	/* such as "Not found..." */
  ED_FILE,	/* such as "/error/404.html" */
  ED_URL	/* such as "http://www.example.com/cgi-bin/404.cgi" */
} errdoc_t;
/* Error document structure */
typedef struct _ErrorDoc {
  errdoc_t		type;	/* error document type */
  int			code;	/* response status code */
  char *		text;	/* document to response */
  struct _ErrorDoc *	next;
} ErrorDoc;

/* Alias type */
typedef enum {
  A_REDIRECT,	/* Redirect */
  A_ALIAS,	/* Alias */
  A_SCRIPT	/* ScriptAlias */
} alias_t;
/* Alias structure */
typedef struct _Alias {
  alias_t		type;		/* alias type */
  char *		fakeName;	/* URI path to file */
  char *		realName;	/* actual path to file */
  size_t		fakeLen;	/* strlen of fakeName */
  size_t		realLen;	/* strlen of realName */
  struct _Alias *	next;
} Alias;

/* Listen Address structure */
#define	SOCKET_UNSECURE	(0)	/* standard connection */
#define	SOCKET_SECURE	(1)	/* via SSL */
typedef struct _ListenAddr {
  char *		ipaddr;	/* ip address */
  char *		port;	/* port number */
  int			secure;	/* secure: 1, unsecure: 0 */
  struct _ListenAddr *	next;
} ListenAddr;

/* Socket other information */
typedef struct {
  unsigned int	secure;	/* secure: 1, unsecure: 0 */
  unsigned int	port;	/* port number */
} SocketInfo;

/* Socket structure */
typedef struct _Socket {
  SocketInfo		info;
  wt_netfd_t *		nsd;
  wt_mutex_t		mlock;
  wt_thread_t *		thread;
  struct _Socket *	next;
} Socket;
#define	wv_socklen(x)	(wt_netfd_socklen((x)->nsd))
#define	wv_sockaddr(x)	(wt_netfd_sockaddr((x)->nsd))

/* Virtual Host structure */
typedef struct _VirtualHost {
  char *	serverName;		/* ServerName */
  char *	serverAdmin;		/* ServerAdmin */
  LogFile	errorLog;		/* ErrorLog */
  LogFile	accessLog;		/* AccessLog */
  unsigned int	timeout;		/* Timeout */
  /**/
  char *	documentRoot;		/* DocumentRoot */
  char *	userDir;		/* UserDir */
  StrList *	directoryIndexes;	/* DirectoryIndexes */
  bool		autoIndexing;		/* AutoIndexing */
  bool		followSymlinks;		/* FollowSymlinks */
  char *	defaultType;		/* DefaultType */
  ErrorDoc *	errorDocTable;		/* ErrorDocument */
  Alias *	aliasTable;		/* Redirect/Alias/ScriptAlias */
  Hash *	mimeTable;		/* AddType */
  /**/
  struct _VirtualHost *	next;
} VirtualHost;

/* Httpd structure */
typedef struct {
  ListenAddr *	listen;		/* Listen */
  unsigned int	maxThread;	/* MaxThread */
  unsigned int	maxRequest;	/* MaxRequest */
  size_t	maxPostSize;	/* MaxPostSize */
  uid_t		user;		/* User */
  gid_t		group;		/* Group */
  int		secureLevel;	/* SecureLevel */
  LogFile	securityLog;	/* SecurityLog */
  char *	pidFile;	/* PidFile */
  pid_t		pid;		/* Wyvern PID */
  char *	serverRoot;	/* ServerRoot */
  char *	tmpDir;		/* TmpDir */
  char *	cacheDir;	/* CacheDir */
  char *	configDir;	/* ConfigDir */
  char *	databaseDir;	/* DatabaseDir */
  char *	typesConfig;	/* typesConfig: mime.types path */
#if	ENABLE_SSL
  char *	sslCertificateFile;	/* SSLCertificateFile */
  char *	sslCertificateKeyFile;	/* SSLCertificateKeyFile */
  char *	sslCipherList;		/* SSLCipherList */
#endif	/* ENABLE_SSL */
  /**/
  VirtualHost *	vhostTable;	/* virtual hosts manager */
  VirtualHost *	host;		/* main server host */
  VirtualHost *	vhost;		/* current virtual host */
  /* for networks */
  Socket *	sockets;	/* sockets list */
  /* flags */
  volatile bool	sighup;
  /* timestamp at wyvern's bootstrap */
  struct timeval	bootTime;
} Httpd;

/*****************************************************************************/

/* Method type */
typedef enum {
  M_NONE,
  M_HEAD,	/* See RFC2616 Section 9.4 (HEAD request) */
  M_GET,	/* See RFC2616 Section 9.3 (GET  request) */
  M_POST,	/* See RFC2616 Section 9.5 (POST request) */
  M_UNKNOWN
} method_t;

/* HTTP version */
typedef enum {
  HTTP_00,	/* unknown version */
  HTTP_09,	/* version 0.9 */
  HTTP_10,	/* version 1.0 */
  HTTP_11	/* version 1.1 */
} http_t;

/* KeepAlive state type */
typedef enum {
  KA_DISABLED,
  KA_CLOSE,
  KA_CONTINUE
} keepalive_t;

typedef struct _Range {
  size_t	start;	/* start position */
  size_t	end;	/* end position */
} Range;

/* HTTP Message structure */
typedef struct {
  /* communication channel */
  wt_netfd_t *	nsd;

  /* socket information */
  SocketInfo	soci;

  /* client's hostname and IP address */
  struct {		/* used by getnameinfo(3) */
    struct sockaddr *	addr;
    char *		port;	/* remote port */
    char *		ip;	/* remote IP address */
    char *		name;	/* remote hostname */
  } client;

  /* keepalive */
  struct {
    keepalive_t		status;		/* keep-alive status */
    unsigned int	timeout;	/* KeepAliveTimeout */
    unsigned int	max;		/* KeepAliveMax */
  } ka;

  /* current virtual host */
  VirtualHost *	vhost;

  /* request/response timestamp */
  struct {
    struct timeval	request;
    struct timeval	response;
  } time;

  /* request */
  struct {
    struct {
      method_t	type;			/* method type */
      char *	name;			/* method name */
    } method;
    char	uri[MAX_URI_LEN + 1];	/* Request URI */
    struct {
      http_t	version;		/* version */
      char *	protocol;		/* protocol */
    } http;
    AAObject *	header;			/* request headers */

    char *	auth_type;		/* AUTH_TYPE */
    char *	path_info;		/* PATH_INFO */
    char *	query_string;		/* QUERY_STRING */
    int		post_fd;		/* File descriptor for POST method */
    char *	post_tmp;		/* Temporary file for POST method */
    size_t	content_length;
  } request;

  /* response */
  struct {
    status_t		status;		/* HTTP response status code */
    const char *	error_mesg;	/* Error message */
    AAObject *		header;		/* response headers */
    size_t		content_length;
  } response;

  /* content */
  struct {
    wt_netfd_t *	nfd;		/* file descriptor */
    off_t		offset;		/* where to begin in the file */
    char *		path;		/* actual path of data file */
    char *		mime;		/* file media type */
    struct stat		status;		/* file status */
    Range *		ranges;		/* range unit for partial request */
    unsigned int	ritems;		/* numbers of range items */
    char *		boundary;	/* boundary for multipart/byteranges */
  } file;

  /* user */
  struct {
    char *	indexDir;	/* path of user directory (i.e. public_html) */
    char *	homeDir;	/* user home directory (Don't call free().) */
    char *	name;		/* owner of UserDir */
    char *	authName;	/* authenticated username */
    char *	identName;	/* identification username */
  } user;

  /* cgi */
  struct {
    pid_t	pid;		/* pid of CGI program */
    int		envno;		/* the number of CGI environments */
    char **	env;		/* environments for CGI program */
  } cgi;

  /* netfd subroutines */
  wt_netfd_t *	(*accept)(wt_netfd_t *, struct sockaddr *, socklen_t *,
			  wt_utime_t);
  int		(*close)(wt_netfd_t *);
  int		(*shutdown)(wt_netfd_t *, int);
  ssize_t	(*read)(wt_netfd_t *, void *, size_t, wt_utime_t);
  ssize_t	(*write)(wt_netfd_t *, const void *, size_t, wt_utime_t);
  ssize_t	(*writev)(wt_netfd_t *, const struct iovec *, int, wt_utime_t);
  ssize_t	(*readline)(wt_netfd_t *, void *, size_t, wt_utime_t);
#if	HAVE_FREEBSD_SENDFILE
  int		(*sendfile)(wt_netfd_t *, wt_netfd_t *, off_t, size_t,
			    struct sf_hdtr *, off_t *, int, wt_utime_t);
#elif	HAVE_LINUX_SENDFILE
  int		(*sendfile)(wt_netfd_t *, wt_netfd_t *, off_t, size_t,
			    void *, off_t *, int, wt_utime_t);
#else	/* No sendfile(2) */
  int		(*sendfile)(wt_netfd_t *, wt_netfd_t *, off_t, size_t,
			    void *, off_t *, int, wt_utime_t);
#endif

} Message;

/* callback interfaces */
typedef int	(*hook_callback_t)(Message * msg);

/*****************************************************************************/

EXTERN Httpd *	httpd;	/* daemon entity */

/*****************************************************************************/

#include "functions.h"

#endif	/* _WYVERN_H */
